#!/bin/env python3

"""
A John Shots - A tool to get the Security Hash Algorightms (SHA) of all file in a given path.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to A John Shots!

Project link:
    https://github.com/funilrys/A-John-Shots

License:
::

    MIT License

    Copyright (c) 2017-2019 Nissar Chababy

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""
# pylint: disable=bad-continuation, too-many-arguments

from re import compile as comp


def combine_dicts(dict_1, dict_2):
    """
    Combine two dictionnaries.

    :param dict_1: The first dict.
    :type dict_1: dict

    :param dict_2: The second dict.
    :type dict_2: dict

    :return: The combined dict.
    :rtype: dict
    """

    result = {}

    for key, value in dict_1.items():
        if key in dict_2.keys():
            if isinstance(dict_2[key], dict):
                result[key] = combine_dicts(value, dict_2.pop(key))
        else:
            result[key] = value
    for key, value in dict_2.items():
        result[key] = value
    return result


class Regex:  # pylint: disable=too-few-public-methods

    """
    A simple implementation ot the python.re package

    Arguments:
        - data: str or list
            The data or a list of data to check.
        - regex: str or list
            The regex or a list or regex.
        - return_data: bool
            - True: Return matched string
            - False: Return False|True
        - group: int
            The group to return.
        - rematch: bool
            Implementation of Bash ${BASH_REMATCH}.
            - True: Returned matched groups into a list format.
        - occurences: int
            The number of occurence to replace.
    """

    def __init__(
        self, data, regex, group=0, occurences=0, rematch=False, return_data=True
    ):
        super(Regex, self).__init__()

        # We initiate the needed variable in order to be usable all over class
        self.data = data
        self.regex = regex

        self.group = group
        self.occurences = occurences
        self.rematch = rematch
        self.return_data = return_data

        # We initiate regex according to self.escape status.
        self.regex = regex

    def match(self, regex=None, data_to_match=None):
        """
        Used to get exploitable result of re.search

        Arguments:
            - data: str
                The data or a list of data to check.
            - regex: str
                The regex or a list or regex.

        Returns:
            list or bool
            - bool: if self.return_data is False
            - list: otherwise
        """

        # We initate this variable which gonna contain the returned data
        result = []

        if not regex:
            regex = self.regex

        if not data_to_match:
            data_to_match = self.data

        # We compile the regex string
        to_match = comp(regex)

        # In case we have to use the implementation of ${BASH_REMATCH} we use
        # re.findall otherwise, we use re.search
        if self.rematch:
            pre_result = to_match.findall(data_to_match)
        else:
            pre_result = to_match.search(data_to_match)

        if self.return_data and pre_result is not None:
            if self.rematch:
                for data in pre_result:
                    if isinstance(data, tuple):
                        result.extend(list(data))
                    else:
                        result.append(data)

                if self.group != 0:
                    return result[self.group]
            else:
                result = pre_result.group(self.group).strip()

            return result

        if not self.return_data and pre_result is not None:
            return True
        return False

    def loop_matching(self):
        """
        This method can be used to perform a loop matching.
        """

        results = []

        if isinstance(self.data, str):
            if isinstance(self.regex, list):
                for exp in self.regex:
                    matched = self.match(regex=exp)

                    try:
                        results.extend(matched)
                    except TypeError:
                        results.append(matched)

                if not self.return_data:
                    if True in results:
                        return True
                    return False
            else:
                return self.match()

        elif isinstance(self.data, list) and isinstance(self.regex, str):
            for string in self.data:
                results.extend(self.match(data_to_match=string))

        return results



#!/bin/env python3

"""
A John Shots - A tool to get the Security Hash Algorightms (SHA) of all file in a given path.

This submodule provide the brain of the module.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to A John Shots!

Project link:
    https://github.com/funilrys/A-John-Shots

License:
::

    MIT License

    Copyright (c) 2017-2019 Nissar Chababy

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""
# pylint: disable=bad-continuation

from os import path
from os import sep as directory_separator
from os import walk

from a_john_shots.defaults import DEFAULT_EXCLUDE
from a_john_shots.hash import Hash
from a_john_shots.helpers import Regex, combine_dicts


class Core:
    """
    The real brain of the module.

    :param file_or_dir_path: The file or directory we are working with.
    :type file_or_dir_path: str

    :param algorithm:
        The SHA to use. Can be one of the following:
        ::

            - all
            - sha1
            - sha224
            - sha384
            - sha51
    :type algorithm: str

    :param search:
        The pattern the filename have to match in order to be
        taken in consideration.
    :type search: str

    :param exclude:
        The pattern the filename have to match in order to be
        excluded.
    :type exclude: str
    """

    def __init__(self, file_or_dir_path, search=None, algorithm="sha512", exclude=None):
        if isinstance(search, str):
            self.search = search
        else:
            self.search = None

        if exclude is None:
            self.exclude = []
        elif isinstance(exclude, list):
            self.exclude = exclude
        else:
            self.exclude = [exclude]

        if file_or_dir_path == ".":
            self.path = f"{file_or_dir_path}{directory_separator}"
        else:
            self.path = file_or_dir_path

        self.algorithm = algorithm
        self.ppath = [self.path]
        self.exclude.extend(DEFAULT_EXCLUDE)

    def get(self):
        """
        Return the :code:`dict` tree representing the given path childs.
        """

        result = {}
        file_hash = {}

        if path.isfile(self.path):
            file_hash = Hash(self.path, self.algorithm, False).get()
            result = self.hierarchy(file_hash)
        elif path.isdir(self.path):
            for root, _, files in walk(self.path):
                for file in files:
                    sub = self.get_childs(root, file)
                    if not self.search:
                        result = combine_dicts(sub, result)
                    elif Regex(file, self.search).match():
                        result = combine_dicts(sub, result)
        else:
            raise Exception("Given file/dir path not found.")

        return result

    def hierarchy(self, hash_of_file):
        """
        Build the hierarchy tree.

        :param hash_of_file: A dict representing a file and its hash.
        :type hash_of_file: dict

        :return: The hierarchy tree.
        :rtype: dict
        """

        result = {}

        for item in self.ppath:
            hierarchy = [x for x in item.split(directory_separator) if x]
            local_result = result

            for funilrys in hierarchy:
                if funilrys == hierarchy[-1]:
                    local_result = local_result.setdefault(funilrys, hash_of_file)
                else:
                    local_result = local_result.setdefault(funilrys, {})

        return result

    def get_childs(self, root, file):
        """
        Get the childs of the current directory.

        :param root: The current root absolute path.
        :type root: str

        :param file: The current file absolute path.
        :type file: str

        :return: A :code:`dict` with the hash of the currently read file.
        :rtype: dict
        """

        result = {}
        path_to_file = f"{root}{directory_separator}{file}"

        if not Regex(path_to_file, self.exclude, return_data=False).loop_matching():
            self.ppath = [x for x in path_to_file.split(self.path) if x]

            file_hash = Hash(path_to_file, self.algorithm).get()
            result = self.hierarchy(file_hash)

        return result



#!/bin/env python3

"""
A John Shots - A tool to get the Security Hash Algorightms (SHA) of all file in a given path.

This is the module entry which provides the API endpoints and the CLI.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to A John Shots!

Project link:
    https://github.com/funilrys/A-John-Shots

License:
::

    MIT License

    Copyright (c) 2017-2019 Nissar Chababy

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""
# pylint: disable=bad-continuation, too-many-arguments
import argparse
from json import dump, dumps

from colorama import Fore, Style
from colorama import init as initiate

from a_john_shots.core import Core
from a_john_shots.defaults import DEFAULT_OUTPUT_FILENAME

VERSION = "2.0.0"


def get(
    file_or_dir_path,
    sha="sha512",
    search=None,
    save=False,
    save_destination=None,
    exclude=None,
):
    """
    Return the SHA representation of the given path children.

    .. note::
        If :code:`file_or_dir_path` is a directory, we will get
        the SHA representation of all children files.

    .. note::
        If :code:`file_or_dir_path` is a file, we will get
        the SHA representation of the given file.

    :param file_or_dir_path: The file or directory we are working with.
    :type file_or_dir_path: str

    :param sha:
        The SHA to use. Can be one of the following:
        ::

            - all
            - sha1
            - sha224
            - sha384
            - sha513
    :type sha: str

    :param search:
        The pattern the filename have to match in order to be
        taken in consideration.
    :type search: str

    :param save:
        Tell the system if we are allowed to save the result into a JSON file.

        .. note::
            The default file name is :code:`faith-slosh.json`.

        .. note::
            Even if this argument is set to :code:`True`, we return the
            dict representation if needed.
    :type save: bool

    :param save_destination: The destination file.
    :type save_destination: str

    :param exclude:
        The pattern the filename have to match in order to be
        excluded.
    :type exclude: str|list
    """

    sha = sha.lower()
    result = Core(file_or_dir_path, search=search, exclude=exclude, algorithm=sha).get()

    if save_destination is None:
        save_destination = DEFAULT_OUTPUT_FILENAME

    if save:
        print(save_destination)
        with open(save_destination, "w") as file:
            dump(result, file, ensure_ascii=False, indent=4, sort_keys=True)

    return result


def command_line():
    """
    Provide the CLI.
    """

    if __name__ == "a_john_shots":
        initiate(True)

        parser = argparse.ArgumentParser(
            description="A John Shots - A tool to get the Security Hash "
            "Algorightms (SHA) of all file in a given path.",
            epilog=f"Crafted with %s by %s"
            % (
                f"{Fore.RED}♥{Fore.RESET}",
                f"{Style.BRIGHT}{Fore.CYAN}Nissar Chababy (Funilrys){Style.RESET_ALL}",
            ),
        )

        parser.add_argument(
            "-a",
            "--algorithm",
            type=str,
            default="sha512",
            help="The SHA algorithm to use. Can be ALL, SHA1, SHA224, SHA384 or SHA512 (default).",
        )

        parser.add_argument(
            "-d",
            "--destination",
            type=str,
            help="Set the file we are going to write the JSON output to.",
        )

        parser.add_argument(
            "-e",
            "--exclude",
            type=str,
            help="The (regex) pattern the filename has to match in order "
            "to be excluded from the output.",
        )

        parser.add_argument(
            "-p", "--path", type=str, help="The file or directory path to read."
        )

        parser.add_argument("--print", action="store_true", help="Print the output.")

        parser.add_argument(
            "-s",
            "--search",
            type=str,
            help="The (regex) pattern the filename has to match in order "
            "to be included into the output.",
        )

        parser.add_argument(
            "--save", action="store_true", help="Save the output into a JSON file."
        )

        parser.add_argument(
            "-v", "--version", action="version", version="%(prog)s " + VERSION
        )

        args = parser.parse_args()

        if args.path:
            if args.print:
                print(
                    dumps(
                        get(
                            args.path,
                            sha=args.algorithm,
                            search=args.search,
                            save=args.save,
                            save_destination=args.destination,
                            exclude=args.exclude,
                        ),
                        ensure_ascii=False,
                        indent=4,
                        sort_keys=True,
                    )
                )
            else:
                get(
                    args.path,
                    sha=args.algorithm,
                    search=args.search,
                    save=args.save,
                    save_destination=args.destination,
                    exclude=args.exclude,
                )



#!/bin/env python3

"""
A John Shots - A tool to get the Security Hash Algorightms (SHA) of all file in a given path.

This submodule provide the hashing logic.

Author:
    Nissar Chababy, @funilrys, contactTATAfunilrysTODTODcom

Contributors:
    Let's contribute to A John Shots!

Project link:
    https://github.com/funilrys/A-John-Shots

License:
::

    MIT License

    Copyright (c) 2017-2019 Nissar Chababy

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
"""
import hashlib
from os import path


class Hash:
    """
    Get and return the hash a file with the given algorithm.

    :param file_path: The absolute path to the file we are working with.
    :type file_path: str

    :param algorithm: The algorithm to use.
    :type algorithm: str

    :param only_hash:
        If :code:`True` return only the desired algorithm
        if algorithm != :code:`all`
    """

    def __init__(self, file_path, algorithm="sha512", only_hash=False):
        self.valid_algorithms = ["all", "md5", "sha1", "sha224", "sha384", "sha512"]

        self.path = file_path
        self.algorithm = algorithm
        self.only_hash = only_hash

    def hash_data(self, algo):
        """
        Get the hash of the globally given file path.

        :param algo: The algorithm to use.
        :type algo: str

        :return: The algo representation of the given file path.
        :rtype: str
        """

        hash_data = getattr(hashlib, algo)()

        with open(self.path, "rb") as file:
            content = file.read()

            hash_data.update(content)
        return hash_data.hexdigest()

    def get(self):
        """
        Return the hash of the given file
        """

        result = {}

        if path.isfile(self.path) and self.algorithm in self.valid_algorithms:
            if self.algorithm == "all":
                del self.valid_algorithms[0]

                for algo in self.valid_algorithms:
                    result[algo] = None
                    result[algo] = self.hash_data(algo)
            else:
                result[self.algorithm] = None
                result[self.algorithm] = self.hash_data(self.algorithm)

        if self.algorithm != "all" and self.only_hash:
            return result[self.algorithm]
        return result
